#include "stdint.h"
#include "stdlib.h"

/* Panel */
#define PANEL_SIZE_MIN         0
#define PANEL_SIZE_MAX         31
#define PANEL_GAME_OVER_SIZE   272
#define PANEL_PAUSE_LENGTH     3000

/* Ball */
#define BALL_DEF_X           0x0010
#define BALL_DEF_Y           0x001C
#define BALL_DEF_DIRECTION   0
#define BALL_DEF_COLOR       COLOR_WHITE
#define BALL_DEF_MOV_SPEED   MOV_SPEED_1

/* Paddle */

// Default cfg
#define PAD_DEF_X        0x000D
#define PAD_DEF_Y        0x001F
#define PAD_DEF_WIDTH    6
#define PAD_DEF_HEIGHT   1
#define PAD_DEF_COLOR    COLOR_RED

/* Brick */
#define BRICK_DEF_WIDTH     4
#define BRICK_DEF_HEIGHT    2
#define BRICK_MAX_ROWS      8
#define BRICK_MAX_COLUMNS   6
#define WALL_SIZE           BRICK_MAX_ROWS * BRICK_MAX_COLUMNS

/* Directions */

// Normal
#define DIR_NORTH       0
#define DIR_NORTHEAST   1
#define DIR_EAST        2
#define DIR_SOUTHEAST   3
#define DIR_SOUTH       4
#define DIR_SOUTHWEST   5
#define DIR_WEST        6
#define DIR_NORTHWEST   7
// Edge
#define DIR_CEILING_1       8
#define DIR_CEILING_2       9
#define DIR_RIGHT_EDGE_1    10
#define DIR_RIGHT_EDGE_2    11
#define DIR_PADDLE_1        12
#define DIR_PADDLE_2        13
#define DIR_LEFT_EDGE_1     14
#define DIR_LEFT_EDGE_2     15

/* Brick collision cases */
#define CASE_UP           16
#define CASE_UP_RIGHT     17
#define CASE_RIGHT        18
#define CASE_DOWN_RIGHT   19
#define CASE_DOWN         20
#define CASE_DOWN_LEFT    21
#define CASE_LEFT         22
#define CASE_UP_LEFT      23

/* Colors */
#define COLOR_RED     0xF800
#define COLOR_GREEN   0x07E0
#define COLOR_BLUE    0x001F
#define COLOR_WHITE   0xFFFF
#define COLOR_BLACK   0x0000

/* Status flags */
#define STATUS_NO_CONTACT       0
#define STATUS_HIT_BRICK        1
#define STATUS_HIT_CEILING      2
#define STATUS_HIT_RIGHT_EDGE   3
#define STATUS_HIT_FLOOR        4
#define STATUS_HIT_LEFT_EDGE    5
#define STATUS_HIT_PADDLE       6

/* Paddle move modes and positions */

// Move modes
#define MOV_MODE_SLIDER_2   0
#define MOV_MODE_TOUCHPAD   1

// Move positions
#define MOV_POSITION_1    0
#define MOV_POSITION_2    1
#define MOV_POSITION_3    2
#define MOV_POSITION_4    3
#define MOV_POSITION_5    4
#define MOV_POSITION_6    5
#define MOV_POSITION_7    6
#define MOV_POSITION_8    7
#define MOV_POSITION_9    8
#define MOV_POSITION_10   9

// Ball move speeds
#define MOV_SPEED_1   50
#define MOV_SPEED_2   40
#define MOV_SPEED_3   30
#define MOV_SPEED_4   20
#define MOV_SPEED_5   10

/* Life support */
#define DEF_NUM_LIVES   4

/* Score values */
#define SCORE_BRICK_VAL   2 

/* Difficulty settings */

// Difficulty thresholds
#define DIF_LEVEL_1   0
#define DIF_LEVEL_2   20
#define DIF_LEVEL_3   30
#define DIF_LEVEL_4   38
#define DIF_LEVEL_5   44
#define DIF_BRONZE    45
#define DIF_SILVER    46
#define DIF_GOLD      47

// Multiplier
#define DIF_MULTIPLIER_1        1
#define DIF_MULTIPLIER_2        2
#define DIF_MULTIPLIER_3        3
#define DIF_MULTIPLIER_4        4
#define DIF_MULTIPLIER_5        5
#define DIF_MULTIPLIER_BRONZE   25   
#define DIF_MULTIPLIER_SILVER   50
#define DIF_MULTIPLIER_GOLD     100

/* Functions */
#define RANDOM( min, max ) ( rand( ) % ( max - min + 1 ) + min )

/** STRUCTURES */

// Object
typedef struct
{
    uint16_t x;
    uint16_t y;
    uint16_t color;

} demo_object_t;

// Ball
typedef struct
{
    uint16_t x;
    uint16_t y;
    uint8_t direction;

} demo_ball_t;

// Boundary object
typedef struct 
{
    uint8_t x_min;
    uint8_t y_min;
    uint8_t x_max;
    uint8_t y_max;
    
} demo_bounds_t;

/** DECLARATIONS */

void demo_generateWall( demo_object_t *wall );
void demo_reset( demo_ball_t *ball, demo_object_t *paddle, demo_object_t *wall );
void demo_moveBall( demo_ball_t *ball );
void demo_movePaddle( demo_object_t *paddle, uint8_t mode );
uint8_t demo_collisionCheck( demo_ball_t *ball, demo_object_t *paddle, demo_object_t *wall );
